from rest_framework import serializers
from phonenumber_field.serializerfields import PhoneNumberField


class PhoneConfirmSendSerializer(serializers.Serializer):
    phone = PhoneNumberField(required=False)


class PhonePreConfirmSendSerializer(serializers.Serializer):
    phone = PhoneNumberField()


class PhoneConfirmCheckCodeSerializer(serializers.Serializer):
    phone_confirmation_code = serializers.CharField(max_length=15)



from django import forms
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext as _

User = get_user_model()


class LoginForm(forms.Form):
    username = forms.CharField()
    password = forms.CharField(widget=forms.PasswordInput)

    def clean(self):
        super(LoginForm, self).clean()
        username = self.cleaned_data.get('username')
        password = self.cleaned_data.get('password')
        valid = False
        if username and password:
            user = User.objects.filter(username=username.lower()).first()

            if not user:
                raise forms.ValidationError(_('User is not found'))

            if user and not user.is_active:
                raise forms.ValidationError(_('User is inactive. You must confirm the registration email address at registration.'))

            valid = user.check_password(password)
        if not valid:
            raise forms.ValidationError(_('Invalid: username / password'))
        return valid



from django.conf import settings
from django.urls import reverse
from django.test import override_settings

from garpix_user.models import UserSession
from garpix_user.tests.utils.settings import GARPIX_USER_SETTINGS
from garpix_user.tests.utils.test_case_mixin import ApiTestMixin
from rest_framework.test import APIClient
from django.utils.translation import ugettext_lazy as _

BASE_GARPIX_USER_SETTINGS = getattr(settings, 'GARPIX_USER', dict())


class UserSessionApiTest(ApiTestMixin):
    def setUp(self):
        self.set_init_data()
        self.client = APIClient()

    def test_create_user_session(self):
        """
        Check creation of user session
        """
        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_user_session-create-user-session'),
            HTTP_ACCEPT='application/json',
            format='json'
        )
        self.assertEqual(response.status_code, 200)
        self.assertIn('session_user', response.json())
        token = response.json()['session_user']['token_number']
        self.assertNotEqual(UserSession.objects.filter(token_number=token).first(), None)

    def test_set_user_to_user_session(self):
        """
        Check creation of user session for authorized user
        """
        self.user_login()
        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_user_session-create-user-session'),
            HTTP_ACCEPT='application/json',
            format='json'
        )
        self.assertEqual(response.status_code, 200)
        self.assertIn('session_user', response.json())
        token = response.json()['session_user']['token_number']
        user_session = UserSession.objects.filter(token_number=token).first()
        self.assertEqual(user_session.user, self.user)

    @override_settings(GARPIX_USER=GARPIX_USER_SETTINGS)
    def test_restore_password(self):
        """
        Check restore password functionality
        """
        if BASE_GARPIX_USER_SETTINGS.get('USE_RESTORE_PASSWORD', False):
            self.User.USERNAME_FIELDS = ('phone', 'email')

            # by email

            self.check_password_restore(self.user_data, 'email')

            self.check_password_restore_less_than_one_minute(self.user_data, 'email')

            # by phone

            self.user = self.User.objects.create_user(**self.new_user_data)

            self.check_password_restore(self.new_user_data, 'phone')

            self.check_password_restore_less_than_one_minute(self.user_data, 'phone')

    def check_password_restore(self, user_data, field):

        # first step - send code

        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_restore_password-send-code'),
            {'username': user_data.get(field)},
            HTTP_ACCEPT='application/json',
            format='json'
        )

        self.assertEqual(response.status_code, 200)
        self.assertIn('token_number', response.json())
        token = response.json()['token_number']
        user_session = UserSession.objects.get(token_number=token)

        # second step - check code

        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_restore_password-check-code'),
            {
                'restore_password_confirm_code': user_session.restore_password_confirm_code
            },
            HTTP_ACCEPT='application/json',
            format='json',
            **{f'HTTP_{UserSession.HEAD_NAME}': token}
        )

        self.assertEqual(response.status_code, 204)

        # third step - update password

        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_restore_password-set-password'),
            {
                'new_password': 'pAsSwOrD12345'
            },
            HTTP_ACCEPT='application/json',
            format='json',
            **{f'HTTP_{UserSession.HEAD_NAME}': token}
        )
        self.assertEqual(response.status_code, 200)
        self.assertDictEqual(response.json(), {"result": _('Password was updated!')})

        # check if user can log in with new password

        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_login'),
            {
                'username': user_data.get(field),
                'password': 'pAsSwOrD12345',
            },
            HTTP_ACCEPT='application/json'
        )
        self.check_login_result(response)

    def check_password_restore_less_than_one_minute(self, user_data, field):
        response = self.client.post(
            reverse('garpix_user:garpix_user_api:api_restore_password-send-code'),
            {'username': user_data.get(field)},
            HTTP_ACCEPT='application/json',
            format='json'
        )

        self.assertEqual(response.status_code, 400)
        self.assertDictEqual(response.json(),
                             {'non_field_errors': [_('Less than 1 minutes has passed since the last request')]})



from django.contrib.auth import get_user_model
from django.db.models import Q


class CustomAuthenticationBackend:
    def authenticate(self, request, username=None, password=None):
        if username is None or password is None:
            return None
        try:
            query = Q()
            for field in get_user_model().USERNAME_FIELDS:
                query |= Q(**{field: username.lower()})
            user = get_user_model().active_objects.get(query)
            pwd_valid = user.check_password(password)
            if pwd_valid:
                return user
            return None
        except get_user_model().DoesNotExist:
            return None

    def get_user(self, user_id):
        try:
            return get_user_model().active_objects.get(pk=user_id)
        except get_user_model().DoesNotExist:
            return None



import string

from django.conf import settings
from django.db import models
from datetime import datetime, timedelta

from django.urls import reverse
from django.utils.translation import ugettext as _
import hashlib
from garpix_utils.string import get_random_string

from garpix_user.mixins.models.confirm.code_length_mixin import CodeLengthMixin
from garpix_user.utils.current_date import set_current_date


class UserEmailConfirmMixin(CodeLengthMixin, models.Model):
    """
    Миксин для подтверждения email
    """

    email_confirmation_code = models.CharField(_("Email confirmation code"), max_length=255, blank=True,
                                               null=True)
    email_code_send_date = models.DateTimeField(_("Code sent date"), blank=True, null=True)
    email_confirmed_date = models.DateTimeField(_("Date email was confirmed"), blank=True, null=True)
    new_email = models.EmailField(_("New email"), blank=True, null=True)

    def send_email_confirmation_link(self):
        from garpix_notify.models import Notify
        from garpix_user.models import UserSession
        from django.contrib.auth import get_user_model

        User = get_user_model()

        model_type = 'user_session' if isinstance(self, UserSession) else 'user'
        hash = str(
            hashlib.sha512(f'{self.email}+{self.email_confirmation_code}'.encode("utf-8")).hexdigest()).lower()
        Notify.send(settings.EMAIL_LINK_CONFIRMATION_EVENT, {
            'confirmation_link': User.confirm_link_redirect_url(model_type, hash)
        }, email=self.new_email)

    def send_email_confirmation_code(self, email=None):
        from django.contrib.auth import get_user_model
        from garpix_user.exceptions import UserRegisteredException, WaitException
        from garpix_notify.models import Notify

        User = get_user_model()

        anybody_have_this_email = User.objects.filter(email=email)
        if isinstance(self, User):
            anybody_have_this_email = anybody_have_this_email.exclude(id=self.id)
        if anybody_have_this_email.count() > 0:
            return UserRegisteredException(field='email', extra_data={
                'field': self._meta.get_field('email').verbose_name.title().lower()})

        if settings.GARPIX_USER.get('TIME_LAST_REQUEST', None):
            if self.email_code_send_date and self.email_code_send_date + timedelta(
                    minutes=settings.GARPIX_USER.get('TIME_LAST_REQUEST')) >= datetime.now(
                self.email_code_send_date.tzinfo):
                return WaitException()

        confirmation_code = get_random_string(self.get_confirm_code_length('email'), string.digits)

        self.new_email = email or self.email

        self.email_confirmation_code = confirmation_code

        self.email_code_send_date = set_current_date()

        self.save()

        if settings.GARPIX_USER.get('USE_EMAIL_LINK_CONFIRMATION', True):
            self.send_email_confirmation_link()
        else:
            Notify.send(settings.EMAIL_CONFIRMATION_EVENT, {
                'confirmation_code': confirmation_code
            }, email=self.new_email)

        return True

    def confirm_email(self, email_confirmation_code):
        from garpix_user.exceptions import IncorrectCodeException, NoTimeLeftException

        if self.email_confirmation_code != email_confirmation_code:
            return IncorrectCodeException(field='email_confirmation_code')

        time_is_up = (datetime.now(
            self.email_code_send_date.tzinfo) - self.email_code_send_date).days > settings.GARPIX_USER.get(
            'CONFIRM_EMAIL_CODE_LIFE_TIME', 6)

        if time_is_up:
            return NoTimeLeftException(field='email_confirmation_code')

        self.is_email_confirmed = True
        self.email = self.new_email
        self.email_confirmation_code = None
        self.email_confirmed_date = set_current_date()
        self.save()
        return True

    @classmethod
    def confirm_email_by_link(cls, hash):
        from garpix_user.exceptions import IncorrectCodeException, NoTimeLeftException

        users_list = cls.objects.all()

        for user in users_list:
            if str(hashlib.sha512(
                    f'{user.email}+{user.email_confirmation_code}'.encode("utf-8")).hexdigest()).lower() == hash:
                time_is_up = (datetime.now(
                    user.email_code_send_date.tzinfo) - user.email_code_send_date).days > settings.GARPIX_USER.get(
                    'CONFIRM_EMAIL_CODE_LIFE_TIME', 6)
                if time_is_up:
                    return False, NoTimeLeftException(field='email_confirmation_code')
                user.is_email_confirmed = True
                user.email = user.new_email or user.email
                user.email_confirmation_code = None
                user.save()
                return True, user

        return False, IncorrectCodeException(field='email_confirmation_code')

    def check_email_confirmation(self):
        return self.is_email_confirmed and self.email_confirmed_date and self.email_confirmed_date + timedelta(
                days=settings.GARPIX_USER.get('EMAIL_CONFIRMATION_LIFE_TIME', 2)) >= datetime.now(
            self.email_confirmed_date.tzinfo)

    @classmethod
    def confirm_link_redirect_url(cls, model_type, hash):
        return reverse('garpix_user:email_confirmation_link', args=[model_type, hash])

    class Meta:
        abstract = True



from django.conf import settings
from django.core.validators import ValidationError
from django.utils.translation import ugettext as _
from rest_framework import serializers


class ModelException(Exception):
    message = ''
    code = 'invalid'
    field = None

    def __init__(self, field=None, extra_data={}):
        self.message = self.message.format(**extra_data)
        self.field = field

    def raise_exception(self, exception_class=ValidationError):
        field = self.field or 'non_field_errors'
        raise exception_class({field: [self.message]}, code=self.code)

    def get_message(self):
        return self.message


class WaitException(ModelException):
    message = settings.GARPIX_USER.get('WAIT_RESPONSE',
                                       _("Less than {min_time} minutes has passed since the last request")).format(
        min_time=settings.GARPIX_USER.get('TIME_LAST_REQUEST', 1))


class UserRegisteredException(ModelException):
    message = settings.GARPIX_USER.get('USER_REGISTERED_RESPONSE',
                                       _("User with such data has been already registered"))


class UserUnregisteredException(ModelException):
    message = settings.GARPIX_USER.get('USER_UNREGISTERED_RESPONSE', _("User with such data has not been registered"))


class IncorrectCodeException(ModelException):
    message = _("Incorrect code")


class NoTimeLeftException(ModelException):
    message = _("Code has expired. Request it again")


class NotAuthenticateException(ModelException):
    message = _("Credentials were not provided")


class NotConfirmedException(ModelException):
    message = _("{field} was not confirmed")


class ValidationErrorSerializer(serializers.Serializer):
    field = serializers.ListField(default=['Error'])



import string
from django.utils.translation import ugettext as _
from django.conf import settings
from django.db import models
from django.utils import timezone
from datetime import datetime, timedelta
from django.contrib.auth import get_user_model

from garpix_utils.string import get_random_string
from phonenumber_field.modelfields import PhoneNumberField

from garpix_user.mixins.models.confirm.code_length_mixin import CodeLengthMixin


class UserPhoneConfirmMixin(CodeLengthMixin, models.Model):
    """
    Миксин для подтверждения номера телефона
    """

    phone_confirmation_code = models.CharField(_('Phone confirmation code'), max_length=15,
                                               blank=True, null=True)
    phone_code_send_date = models.DateTimeField(_("Code sent date"), blank=True, null=True)
    phone_confirmed_date = models.DateTimeField(_("Date phone was confirmed"), blank=True, null=True)
    new_phone = PhoneNumberField(_("New phone number"), unique=True, blank=True, null=True)

    def send_phone_confirmation_code(self, phone=None):
        from garpix_user.exceptions import UserRegisteredException, WaitException
        from garpix_notify.models import Notify

        User = get_user_model()

        anybody_have_this_phone = User.objects.filter(phone=phone)
        if isinstance(self, User):
            anybody_have_this_phone = anybody_have_this_phone.exclude(id=self.id)
        if anybody_have_this_phone.count() > 0:
            return UserRegisteredException(field='phone', extra_data={'field': self._meta.get_field('phone').verbose_name.title().lower()})

        if settings.GARPIX_USER.get('TIME_LAST_REQUEST', None):
            if self.phone_code_send_date and self.phone_code_send_date + timedelta(
                    minutes=settings.GARPIX_USER.get('TIME_LAST_REQUEST')) >= datetime.now(self.phone_code_send_date.tzinfo):
                return WaitException()

        confirmation_code = get_random_string(self.get_confirm_code_length('phone'), string.digits)

        self.new_phone = phone or self.phone

        self.phone_confirmation_code = confirmation_code

        try:
            self.phone_code_send_date = timezone.now()
        except Exception:
            self.phone_code_send_date = datetime.now()

        self.save()

        Notify.send(settings.PHONE_CONFIRMATION_EVENT, {
            'confirmation_code': confirmation_code
        }, phone=self.new_phone)

        return True

    def confirm_phone(self, phone_confirmation_code):
        from garpix_user.exceptions import IncorrectCodeException, NoTimeLeftException

        if self.phone_confirmation_code != phone_confirmation_code:
            return IncorrectCodeException(field='phone_confirmation_code')

        time_is_up = (datetime.now(
            self.phone_code_send_date.tzinfo) - self.phone_code_send_date).seconds / 60 > settings.GARPIX_USER.get('CONFIRM_PHONE_CODE_LIFE_TIME', 6)

        if time_is_up:
            return NoTimeLeftException(field='phone_confirmation_code')

        self.is_phone_confirmed = True
        self.phone_confirmation_code = None
        self.phone = self.new_phone
        self.save()
        return True

    def check_phone_confirmation(self):
        return self.is_phone_confirmed and self.phone_confirmed_date and self.phone_confirmed_date + timedelta(
                days=settings.GARPIX_USER.get('PHONE_CONFIRMATION_LIFE_TIME', 2)) >= datetime.now(
            self.phone_confirmed_date.tzinfo)

    class Meta:
        abstract = True



import json
from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.mixins import UserPassesTestMixin
from django.http import JsonResponse, HttpResponseNotAllowed
from django.shortcuts import redirect
from django.views.generic.base import RedirectView
from django.views.generic import FormView
from django.http import HttpResponse
from garpix_user.forms import LoginForm


class LogoutView(RedirectView):
    def get_redirect_url(self):
        logout(self.request)
        return self.url


class LoginView(UserPassesTestMixin, FormView):
    http_method_names = ['post']

    def http_method_not_allowed(self, request, *args, **kwargs):
        if self.request.accepts('text/html'):
            return redirect(self.request.GET.get('next', '/'))
        return HttpResponseNotAllowed(self._allowed_methods())

    def test_func(self):
        return not self.request.user.is_authenticated

    def handle_no_permission(self):
        if self.request.accepts('text/html'):
            return redirect(self.request.GET.get('next', '/'))
        return HttpResponse({"__all__": [_("Your are already authenticated")]}, content_type='application/json',
                            status=403)

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        if not kwargs.get('data'):
            kwargs['data'] = json.loads(getattr(self.request, 'body') or b'{}')
        return kwargs

    @staticmethod
    def get_form_class(**kwargs):
        return LoginForm

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        return context

    def form_valid(self, form):
        request = self.request
        data = form.data
        username = data.get('username')
        password = data.get('password')
        user = authenticate(request, username=username.lower(), password=password)
        if user:
            user.set_user_session(request)
        login(request, user)
        if self.request.accepts('text/html'):
            return redirect(request.GET.get('next', '/'))
        return JsonResponse({'success': True})

    def form_invalid(self, form):
        if self.request.accepts('text/html'):
            return self.render_to_response(self.get_context_data(form=form))
        return HttpResponse(json.dumps(form.errors), content_type='application/json', status=400)



from garpix_utils.models import AdminDeleteMixin
from django.contrib.auth.admin import UserAdmin as BaseUserAdmin
from django.utils.translation import gettext_lazy as _


class UserAdmin(AdminDeleteMixin, BaseUserAdmin):
    change_form_template = "garpix_user/send_confirm.html"

    fieldsets = (
        (None, {'fields': ('username', 'password')}),
        (_('Personal info'), {'fields': ('first_name', 'last_name', 'email', 'phone')}),
        (_('Permissions'), {
            'fields': ('is_active', 'is_deleted', 'is_staff', 'is_superuser', 'groups', 'user_permissions'),
        }),
        (_('Viber'), {
            'fields': ('viber_chat_id', 'viber_secret_key',)
        }),
        (_('Telegram'), {
            'fields': ('telegram_chat_id', 'telegram_secret', 'get_telegram_connect_user_help'),
        }),
        (_('Confim information'), {
            'fields': ('is_email_confirmed', 'email_confirmation_code', 'is_phone_confirmed', 'phone_confirmation_code'),
        }),
    )
    readonly_fields = ['telegram_secret', 'get_telegram_connect_user_help'] + list(BaseUserAdmin.readonly_fields)



from setuptools import setup, find_packages
from os import path

here = path.join(path.abspath(path.dirname(__file__)), 'garpix_user')

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='garpix_user',
    version='3.7.1',
    description='',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/garpixcms/garpix_user',
    author='Garpix LTD',
    author_email='info@garpix.com',
    license='MIT',
    packages=find_packages(exclude=['testproject', 'testproject.*']),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'Django >= 3.1, < 4',
        'djangorestframework >= 3.8',
        'django-oauth-toolkit >= 1.1.2',
        'social-auth-app-django >= 2.1.0',
        'social-auth-core == 4.3.0',
        'django-rest-framework-social-oauth2 >= 1.1.0',
        'django-phonenumber-field-for-garpix_user >= 8.0.0',
        'garpix-notify >= 5.12.5',
        'garpix-utils >= 1.5.1',
        'drf-spectacular >= 0.24.2'
    ]
)



from drf_spectacular.utils import extend_schema
from rest_framework import parsers, renderers

from garpix_user.models.access_token import AccessToken as Token
from garpix_user.serializers.auth_token_serializer import AuthTokenSerializer
from rest_framework.response import Response
from rest_framework.views import APIView
from django.conf import settings
from garpix_user.models.refresh_token import RefreshToken
from garpix_user.utils.drf_spectacular import user_session_token_header_parameter


@extend_schema(
    parameters=[
        user_session_token_header_parameter()
    ]
)
class ObtainAuthToken(APIView):
    throttle_classes = ()
    permission_classes = ()
    parser_classes = (parsers.FormParser, parsers.MultiPartParser, parsers.JSONParser,)
    renderer_classes = (renderers.JSONRenderer,)
    serializer_class = AuthTokenSerializer

    def post(self, request, *args, **kwargs):
        serializer = self.serializer_class(data=request.data,
                                           context={'request': request})
        serializer.is_valid(raise_exception=True)
        user = serializer.validated_data['user']
        token = Token.objects.create(user=user)
        refresh_token = RefreshToken.objects.create(user=user)
        return Response({
            'access_token': token.key,
            'refresh_token': refresh_token.key,
            'token_type': 'Bearer',
            'access_token_expires': settings.GARPIX_ACCESS_TOKEN_TTL_SECONDS,
            'refresh_token_expires': settings.GARPIX_REFRESH_TOKEN_TTL_SECONDS,
        })


obtain_auth_token = ObtainAuthToken.as_view()



from django.conf import settings
from django.contrib.auth import get_user_model
from django.db import models, transaction
from django.db.models import Q
from django.utils.translation import gettext as _
from garpix_utils.string import get_random_string
import string
from datetime import datetime, timezone, timedelta

from garpix_user.exceptions import NotConfirmedException
from garpix_user.exceptions import IncorrectCodeException, NoTimeLeftException, WaitException, UserUnregisteredException


class RestorePasswordMixin(models.Model):
    class RESTORE_BY(models.TextChoices):
        PHONE = ('phone', _('Phone number'))
        EMAIL = ('email', _('Email'))

    restore_password_confirm_code = models.CharField(_('Restore password code'),
                                                     max_length=15, null=True, blank=True)
    is_restore_code_confirmed = models.BooleanField(_('Restore code confirmed'), blank=True, default=False)
    restore_date = models.DateTimeField(_('Restore code sent date'), null=True)
    restore_by = models.CharField(choices=RESTORE_BY.choices, default=RESTORE_BY.EMAIL, max_length=5)

    def _check_and_get_user(self, username):

        User = get_user_model()

        USERNAME_FIELDS = getattr(User, 'USERNAME_FIELDS', ('email',))

        user_data = Q()

        for field in USERNAME_FIELDS:
            user_data |= Q(**{field: username, f'is_{field}_confirmed': True})

        if user := User.active_objects.filter(user_data).first():
            return True, user

        return False, UserUnregisteredException(field='username',
                                                extra_data={'value': username})

    def _check_request_time(self):

        if (TIME_LAST_REQUEST := settings.GARPIX_USER.get('TIME_LAST_REQUEST', 1)) and (
                restore_date := self.restore_date):
            if restore_date + timedelta(minutes=TIME_LAST_REQUEST) >= datetime.now(restore_date.tzinfo):
                return False, WaitException()

        return True, None

    def _time_is_up(self):
        datediff = datetime.now(self.restore_date.tzinfo) - self.restore_date

        if self.restore_by == self.RESTORE_BY.EMAIL:
            return datediff.days > settings.GARPIX_USER.get(
                'CONFIRM_EMAIL_CODE_LIFE_TIME', 6)
        return datediff.seconds / 60 > settings.GARPIX_USER.get(
            'CONFIRM_PHONE_CODE_LIFE_TIME', 6)

    def send_restore_code(self, username=None):
        from garpix_notify.models import Notify

        result, data = self._check_and_get_user(username)
        if not result:
            return result, data
        user = data
        result, error = self._check_request_time()
        if not result:
            return result, error

        confirmation_code = get_random_string(settings.GARPIX_USER.get('CONFIRM_CODE_LENGTH', 6), string.digits)

        self.restore_password_confirm_code = confirmation_code
        self.restore_date = datetime.now(timezone.utc)
        self.is_restore_code_confirmed = False

        if user.email == username and 'email' in user.USERNAME_FIELDS:
            Notify.send(settings.RESTORE_PASSWORD_EMAIL_EVENT, {
                'user': user,
                'restore_code': self.restore_password_confirm_code
            }, email=user.email)
            self.restore_by = self.RESTORE_BY.EMAIL
            self.email = username
        elif 'phone' in user.USERNAME_FIELDS:
            Notify.send(settings.RESTORE_PASSWORD_PHONE_EVENT, {
                'user': user,
                'restore_code': self.restore_password_confirm_code
            }, phone=user.phone)
            self.restore_by = self.RESTORE_BY.PHONE
            self.phone = username

        self.save()

        return True, None

    def check_restore_code(self, username, restore_password_confirm_code=None):

        if getattr(self,
                   self.restore_by) != username or self.restore_password_confirm_code != restore_password_confirm_code:
            return False, IncorrectCodeException(field='restore_password_confirm_code')

        time_is_up = self._time_is_up()

        if time_is_up:
            return False, NoTimeLeftException(field='restore_password_confirm_code')

        self.is_restore_code_confirmed = True
        self.save()

        return True, None

    def restore_password(self, new_password, username, restore_password_confirm_code=None):
        User = get_user_model()

        USERNAME_FIELDS = getattr(User, 'USERNAME_FIELDS', ('email',))

        field_name = '/'.join([User._meta.get_field(
            field).verbose_name.title().lower() for field in USERNAME_FIELDS]).rstrip('/')

        if self.is_restore_code_confirmed and restore_password_confirm_code == self.restore_password_confirm_code and getattr(
                self, self.restore_by) == username:

            time_is_up = self._time_is_up()

            if time_is_up:
                return False, NoTimeLeftException(field='restore_password_confirm_code')

            result, data = self._check_and_get_user(username)
            if not result:
                return result, data

            with transaction.atomic():
                user = data
                user.set_password(new_password)
                user.save()
                self.is_restore_code_confirmed = False
                self.save()
            return True, None
        return False, NotConfirmedException(
            extra_data={'field': field_name})

    class Meta:
        abstract = True
